/************************************************************************************
beego RT-Component
Copyright (c) 2010, Kyoto Uni.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

// -*- C++ -*-
/*!
 * @file  beego.cpp * @brief component * $Date$ 
 *
 * $Id$ 
 */
#include "beego.h"

// Module specification
// <rtc-template block="module_spec">
static const char* beego_spec[] =
  {
    "implementation_id", "beego",
    "type_name",         "beego",
    "description",       "component for controlling beego",
    "version",           "1.0.0",
    "vendor",            "Kyoto-u",
    "category",          "device",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "5",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
	"conf.default.str_beego_port","/dev/ttyUSB0",
	//"conf.default.str_urg_port","COM2",
	//"conf.default.bool_use_urg","false",
    "conf.default.double_x_init", "0",
    "conf.default.double_y_init", "0",
    "conf.default.double_theta_init", "0",
    ""
  };
// </rtc-template>


beego::beego(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inputIn("input", m_input),
    m_odometryOut("odometry", m_odometry),
    m_scanOut("scan", m_scan)
    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("input", m_inputIn);

  // Set OutPort buffer
  registerOutPort("odometry", m_odometryOut);
  //registerOutPort("scan", m_scanOut);
  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

}

beego::~beego()
{
}


RTC::ReturnCode_t beego::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("str_beego_port",m_str_beego_port,"/dev/ttyUSB0");
  //bindParameter("str_urg_port",m_str_urg_port,"COM2");
  //bindParameter("bool_use_urg",m_bool_use_urg,"0");
  bindParameter("double_x_init", m_double_x_init, "0.0");
  bindParameter("double_y_init", m_double_y_init, "0.0");
  bindParameter("double_theta_init", m_double_theta_init, "0.0");
  m_configsets.update("default");

  // </rtc-template>
  return RTC::RTC_OK;
}



RTC::ReturnCode_t beego::onFinalize()
{
  setVelocityCtrl(run,0,deg(0));
  run.disconnect();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t beego::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t beego::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t beego::onActivated(RTC::UniqueId ec_id)
{

	//Get device name(beego and URG)
	char dname_beego[20];
	//char dname_urg[20];
	if(m_str_beego_port.size()>=1)
	{
		strcpy(dname_beego, m_str_beego_port.c_str());
	}
	//if(m_str_urg_port.size()>=1){
	//	strcpy(dname_urg, m_str_urg_port.c_str());
	//}

	//Connect beego
	if(!run.isConnected())
	{
		if(run.connect(dname_beego,115200) <0 )
		{
			std::cout<<run.what()<<std::endl;
			return RTC::RTC_ERROR;
		}
	}

	/*
    //Connect URG
	if(m_bool_use_urg == 1 && !urg.isConnected())
	{		
		if(urg.connect(dname_urg,115200,URG::AutoCapture) < 0)
		{
			std::cout<< urg.what() << std::endl;
			return RTC::RTC_ERROR;
		}
	}
	*/

	//Adjust initial position
	run.adjustRunPosition(VXV::Position(m_double_x_init*1000,m_double_y_init*1000,VXV::rad(m_double_theta_init)));

    //if(urg.isConnected())urg.setOwnCrdToObject(&run);
	run.adjustSubTreeTicks(0);
	pre_times = 0;

	return RTC::RTC_OK;
}

RTC::ReturnCode_t beego::onDeactivated(RTC::UniqueId ec_id)
{
  //Send stop signal
  setVelocityCtrl(run,0,deg(0));

  //Disconnect
  if(run.isConnected())run.disconnect();

  //if(urg.isConnected())urg.disconnect();
  return RTC::RTC_OK;
}

RTC::ReturnCode_t beego::onExecute(RTC::UniqueId ec_id)
{
  //Input 
  if(m_inputIn.isNew())
  {
	  m_inputIn.read();
	  double v = m_input.data.vx*1000;
	  double w = m_input.data.va;
	  setVelocityCtrl(run,v,rad(w));
  }

  /*
  //OutPut(URG, if define )
  if(urg.isConnected())
  {
	  if(urg.capture() > 0)
	  {
		  int times = urg.getCaptureTimes();
		  if(times > pre_times)
		  {
			pre_times = times;

			//Output data
			int ticks = urg.crd_ticks;
			m_scan.tm.sec = ticks/1000;
			m_scan.tm.nsec= (ticks - ticks/1000)*1000;
			m_scan.data.length(sizeof(urg.length)/sizeof(urg.length[0]));
			for(int i= 0;i<(int)sizeof(urg.length)/sizeof(urg.length[0]);i++)
			{
				m_scan.data[i] = urg.length[i];	
			}
			m_scanOut.write();
		  }
	  }  
  }
  */

  //Outout(Odometry)
  VXV::Position pos =  run.getRunPosition();
  int ticks                   = run.crd_ticks;
  m_odometry.tm.sec           = ticks/1000;
  m_odometry.tm.nsec          = (ticks - (ticks/1000)*1000)*1000;
  m_odometry.data.position.x  = (double)pos.x/1000;
  m_odometry.data.position.y  = (double)pos.y/1000;
  m_odometry.data.heading     = (double)pos.zt.to_rad();
  m_odometryOut.write();

  //std::cout<<m_odometry.tm.sec<<" "<<m_odometry.tm.nsec<<std::endl;
  printf("<odometry> : time %.3lf[sec], x %.3lf[m],y %.3lf[m],theta %.3lf[rad]\r",(double)ticks/1000,(double)pos.x/1000,(double)pos.y/1000,pos.zt.to_rad());

  return RTC::RTC_OK;
}


RTC::ReturnCode_t beego::onAborting(RTC::UniqueId ec_id)
{
  setVelocityCtrl(run,0,deg(0));
  run.disconnect();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t beego::onError(RTC::UniqueId ec_id)
{
  setVelocityCtrl(run,0,deg(0));
  run.disconnect();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t beego::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t beego::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t beego::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void beegoInit(RTC::Manager* manager)
  {
    coil::Properties profile(beego_spec);
    manager->registerFactory(profile,
                             RTC::Create<beego>,
                             RTC::Delete<beego>);
  }
  
};



